@@ -11,13 +11,12 @@ use glob::Pattern;
 use log::{debug, info};
 
 pub struct Linter {
-    pub name: String,
+    pub code: String,
     pub include_patterns: Vec<Pattern>,
     pub exclude_patterns: Vec<Pattern>,
     pub commands: Vec<String>,
     pub init_commands: Option<Vec<String>>,
     pub config_path: AbsPath,
-    pub bypass_matched_file_filter: bool,
 }
 
 fn matches_relative_path(base: &Path, from: &Path, pattern: &Pattern) -> bool {
@@ -70,63 +69,82 @@ impl Linter {
 
         debug!(
             "Running linter {}: {} {}",
-            self.name,
+            self.code,
             program[0],
             arguments.join(" ")
         );
 
         let start = std::time::Instant::now();
-        let command = Command::new(&program[0]).args(arguments).output()?;
-        debug!("Linter {} took: {:?}", self.name, start.elapsed());
+        let command = Command::new(&program[0])
+            .args(&arguments)
+            .output()
+            .with_context(|| {
+                format!(
+                    "Failed to execute linter command {} with args: {:?}",
+                    program[0], arguments
+                )
+            })?;
+        debug!("Linter {} took: {:?}", self.code, start.elapsed());
 
         if !&command.status.success() {
-            let stderr = std::str::from_utf8(&command.stderr)?.to_owned();
-            return Err(anyhow::Error::msg(format!(
-                "lint adapter for '{}' failed with non-zero exit code",
-                self.name
-            )))
-            .with_context(|| stderr);
+            let stderr = std::str::from_utf8(&command.stderr)?;
+            let stdout = std::str::from_utf8(&command.stdout)?;
+            bail!(
+                "Linter command failed with non-zero exit code.\n\
+                 STDERR:\n{}\n\nSTDOUT:{}\n",
+                stderr,
+                stdout,
+            );
         }
         let stdout_str = std::str::from_utf8(&command.stdout)?;
-        let lints = stdout_str
+        stdout_str
             .split("\n")
             .filter(|line| !line.is_empty())
-            .map(|line| LintMessage::from_json(line))
+            .map(|line| serde_json::from_str(line).map_err(|a| anyhow::Error::msg(a.to_string())))
             .collect::<Result<Vec<LintMessage>>>()
             .context(format!(
                 "Failed to deserialize output for lint adapter: '{}'",
-                self.name
-            ))?;
-        if self.bypass_matched_file_filter {
-            Ok(lints)
-        } else {
-            Ok(lints
-                .into_iter()
-                .filter(|lint| {
-                    if let Some(path) = &lint.path {
-                        matched_files.contains(path)
-                    } else {
-                        // Always display lints without a path.
-                        true
-                    }
-                })
-                .collect())
-        }
+                self.code
+            ))
     }
 
-    pub fn run(&self, files: &Vec<AbsPath>) -> Result<Vec<LintMessage>> {
+    pub fn run(&self, files: &Vec<AbsPath>) -> Vec<LintMessage> {
         let matches = self.get_matches(&files);
-        debug!("Linter '{}' matched files: {:#?}", self.name, matches);
+        debug!("Linter '{}' matched files: {:#?}", self.code, matches);
         if matches.is_empty() {
-            return Ok(Vec::new());
+            return Vec::new();
+        }
+        // Wrap the command in a Result to ensure uniform error handling.
+        // This way, linters are guaranteed to exit cleanly, and any issue will
+        // be reported using the same mechanism that we use to report regular
+        // lint errors.
+        match self.run_command(matches) {
+            Err(e) => {
+                let err_lint = LintMessage {
+                    path: None,
+                    line: None,
+                    char: None,
+                    code: self.code.clone(),
+                    severity: crate::lint_message::LintSeverity::Error,
+                    name: "Linter failed".to_string(),
+                    description: Some(format!(
+                        "Linter failed. This a bug, please file an issue against \
+                                 the linter maintainer.\n\nCONTEXT:\n{}",
+                        e
+                    )),
+                    original: None,
+                    replacement: None,
+                };
+                return vec![err_lint];
+            }
+            Ok(messages) => messages,
         }
-        self.run_command(matches)
     }
 
     pub fn init(&self, dry_run: bool) -> Result<()> {
         match &self.init_commands {
             Some(init_commands) => {
-                info!("Initializing linter: '{}'", self.name);
+                info!("Initializing linter: '{}'", self.code);
                 if init_commands.is_empty() {
                     return Ok(());
                 }
@@ -143,7 +161,7 @@ impl Linter {
                 if !status.success() {
                     bail!(
                         "lint initializer for '{}' failed with non-zero exit code",
-                        self.name
+                        self.code
                     );
                 }
                 Ok(())
